@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2007-2010 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2007-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -30,6 +30,7 @@
 
 #include <string.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 
 #include <packagekit-glib2/pk-common.h>
 #include <packagekit-glib2/pk-enum.h>
@@ -46,6 +47,7 @@ static const PkEnumMatch enum_exit[] = {
 	{PK_EXIT_ENUM_NEED_UNTRUSTED,		"need-untrusted"},
 	{PK_EXIT_ENUM_CANCELLED_PRIORITY,	"cancelled-priority"},
 	{PK_EXIT_ENUM_SKIP_TRANSACTION,		"skip-transaction"},
+	{PK_EXIT_ENUM_REPAIR_REQUIRED,		"repair-required"},
 	{0, NULL}
 };
 
@@ -65,7 +67,6 @@ static const PkEnumMatch enum_status[] = {
 	{PK_STATUS_ENUM_OBSOLETE,		"obsolete"},
 	{PK_STATUS_ENUM_DEP_RESOLVE,		"dep-resolve"},
 	{PK_STATUS_ENUM_SIG_CHECK,		"sig-check"},
-	{PK_STATUS_ENUM_ROLLBACK,		"rollback"},
 	{PK_STATUS_ENUM_TEST_COMMIT,		"test-commit"},
 	{PK_STATUS_ENUM_COMMIT,			"commit"},
 	{PK_STATUS_ENUM_REQUEST,		"request"},
@@ -93,12 +94,14 @@ static const PkEnumMatch enum_status[] = {
 static const PkEnumMatch enum_role[] = {
 	{PK_ROLE_ENUM_UNKNOWN,				"unknown"},	/* fall though value */
 	{PK_ROLE_ENUM_CANCEL,				"cancel"},
-	{PK_ROLE_ENUM_GET_DEPENDS,			"get-depends"},
+	{PK_ROLE_ENUM_DEPENDS_ON,			"depends-on"},
 	{PK_ROLE_ENUM_GET_DETAILS,			"get-details"},
+	{PK_ROLE_ENUM_GET_DETAILS_LOCAL,		"get-details-local"},
+	{PK_ROLE_ENUM_GET_FILES_LOCAL,			"get-files-local"},
 	{PK_ROLE_ENUM_GET_FILES,			"get-files"},
 	{PK_ROLE_ENUM_GET_PACKAGES,			"get-packages"},
 	{PK_ROLE_ENUM_GET_REPO_LIST,			"get-repo-list"},
-	{PK_ROLE_ENUM_GET_REQUIRES,			"get-requires"},
+	{PK_ROLE_ENUM_REQUIRED_BY,			"required-by"},
 	{PK_ROLE_ENUM_GET_UPDATE_DETAIL,		"get-update-detail"},
 	{PK_ROLE_ENUM_GET_UPDATES,			"get-updates"},
 	{PK_ROLE_ENUM_INSTALL_FILES,			"install-files"},
@@ -108,25 +111,20 @@ static const PkEnumMatch enum_role[] = {
 	{PK_ROLE_ENUM_REMOVE_PACKAGES,			"remove-packages"},
 	{PK_ROLE_ENUM_REPO_ENABLE,			"repo-enable"},
 	{PK_ROLE_ENUM_REPO_SET_DATA,			"repo-set-data"},
+	{PK_ROLE_ENUM_REPO_REMOVE,			"repo-remove"},
 	{PK_ROLE_ENUM_RESOLVE,				"resolve"},
-	{PK_ROLE_ENUM_ROLLBACK,				"rollback"},
 	{PK_ROLE_ENUM_SEARCH_DETAILS,			"search-details"},
 	{PK_ROLE_ENUM_SEARCH_FILE,			"search-file"},
 	{PK_ROLE_ENUM_SEARCH_GROUP,			"search-group"},
 	{PK_ROLE_ENUM_SEARCH_NAME,			"search-name"},
 	{PK_ROLE_ENUM_UPDATE_PACKAGES,			"update-packages"},
-	{PK_ROLE_ENUM_UPDATE_SYSTEM,			"update-system"},
 	{PK_ROLE_ENUM_WHAT_PROVIDES,			"what-provides"},
 	{PK_ROLE_ENUM_ACCEPT_EULA,			"accept-eula"},
 	{PK_ROLE_ENUM_DOWNLOAD_PACKAGES,		"download-packages"},
 	{PK_ROLE_ENUM_GET_DISTRO_UPGRADES,		"get-distro-upgrades"},
 	{PK_ROLE_ENUM_GET_CATEGORIES,			"get-categories"},
 	{PK_ROLE_ENUM_GET_OLD_TRANSACTIONS,		"get-old-transactions"},
-	{PK_ROLE_ENUM_SIMULATE_INSTALL_FILES,		"simulate-install-files"},
-	{PK_ROLE_ENUM_SIMULATE_INSTALL_PACKAGES,	"simulate-install-packages"},
-	{PK_ROLE_ENUM_SIMULATE_REMOVE_PACKAGES,		"simulate-remove-packages"},
-	{PK_ROLE_ENUM_SIMULATE_UPDATE_PACKAGES,		"simulate-update-packages"},
-	{PK_ROLE_ENUM_UPGRADE_SYSTEM,			"upgrade-system"},
+	{PK_ROLE_ENUM_REPAIR_SYSTEM,			"repair-system"},
 	{0, NULL}
 };
 
@@ -197,6 +195,8 @@ static const PkEnumMatch enum_error[] = {
 	{PK_ERROR_ENUM_INSTALL_ROOT_INVALID,	"install-root-invalid"},
 	{PK_ERROR_ENUM_CANNOT_FETCH_SOURCES,	"cannot-fetch-sources"},
 	{PK_ERROR_ENUM_CANCELLED_PRIORITY,	"cancelled-priority"},
+	{PK_ERROR_ENUM_UNFINISHED_TRANSACTION,	"unfinished-transaction"},
+	{PK_ERROR_ENUM_LOCK_REQUIRED,		"lock-required"},
 	{0, NULL}
 };
 
@@ -211,27 +211,6 @@ static const PkEnumMatch enum_restart[] = {
 	{0, NULL}
 };
 
-static const PkEnumMatch enum_message[] = {
-	{PK_MESSAGE_ENUM_UNKNOWN,		"unknown"},	/* fall though value */
-	{PK_MESSAGE_ENUM_BROKEN_MIRROR,		"broken-mirror"},
-	{PK_MESSAGE_ENUM_CONNECTION_REFUSED,	"connection-refused"},
-	{PK_MESSAGE_ENUM_PARAMETER_INVALID,	"parameter-invalid"},
-	{PK_MESSAGE_ENUM_PRIORITY_INVALID,	"priority-invalid"},
-	{PK_MESSAGE_ENUM_BACKEND_ERROR,		"backend-error"},
-	{PK_MESSAGE_ENUM_DAEMON_ERROR,		"daemon-error"},
-	{PK_MESSAGE_ENUM_CACHE_BEING_REBUILT,	"cache-being-rebuilt"},
-	{PK_MESSAGE_ENUM_UNTRUSTED_PACKAGE,	"untrusted-package"},
-	{PK_MESSAGE_ENUM_NEWER_PACKAGE_EXISTS,	"newer-package-exists"},
-	{PK_MESSAGE_ENUM_COULD_NOT_FIND_PACKAGE,	"could-not-find-package"},
-	{PK_MESSAGE_ENUM_CONFIG_FILES_CHANGED,	"config-files-changed"},
-	{PK_MESSAGE_ENUM_PACKAGE_ALREADY_INSTALLED, "package-already-installed"},
-	{PK_MESSAGE_ENUM_AUTOREMOVE_IGNORED, "autoremove-ignored"},
-	{PK_MESSAGE_ENUM_REPO_METADATA_DOWNLOAD_FAILED, "repo-metadata-download-failed"},
-	{PK_MESSAGE_ENUM_REPO_FOR_DEVELOPERS_ONLY, "repo-for-developers-only"},
-	{PK_MESSAGE_ENUM_OTHER_UPDATES_HELD_BACK, "other-updates-held-back"},
-	{0, NULL}
-};
-
 static const PkEnumMatch enum_filter[] = {
 	{PK_FILTER_ENUM_UNKNOWN,		"unknown"},	/* fall though value */
 	{PK_FILTER_ENUM_NONE,			"none"},
@@ -259,6 +238,8 @@ static const PkEnumMatch enum_filter[] = {
 	{PK_FILTER_ENUM_NOT_COLLECTIONS,	"~collections"},
 	{PK_FILTER_ENUM_APPLICATION,		"application"},
 	{PK_FILTER_ENUM_NOT_APPLICATION,	"~application"},
+	{PK_FILTER_ENUM_DOWNLOADED,		"downloaded"},
+	{PK_FILTER_ENUM_NOT_DOWNLOADED,		"~downloaded"},
 	{0, NULL}
 };
 
@@ -351,21 +332,6 @@ static const PkEnumMatch enum_upgrade[] = {
 	{0, NULL}
 };
 
-static const PkEnumMatch enum_provides[] = {
-	{PK_PROVIDES_ENUM_UNKNOWN,		"unknown"},	/* fall though value */
-	{PK_PROVIDES_ENUM_ANY,			"any"},
-	{PK_PROVIDES_ENUM_MODALIAS,		"modalias"},
-	{PK_PROVIDES_ENUM_CODEC,		"codec"},
-	{PK_PROVIDES_ENUM_MIMETYPE,		"mimetype"},
-	{PK_PROVIDES_ENUM_HARDWARE_DRIVER,	"driver"},
-	{PK_PROVIDES_ENUM_FONT,			"font"},
-	{PK_PROVIDES_ENUM_POSTSCRIPT_DRIVER,	"postscript-driver"},
-	{PK_PROVIDES_ENUM_PLASMA_SERVICE,	"plasma-service"},
-	{PK_PROVIDES_ENUM_SHARED_LIB,		"shared-library"},
-	{PK_PROVIDES_ENUM_PYTHON,		"python-module"},
-	{0, NULL}
-};
-
 static const PkEnumMatch enum_network[] = {
 	{PK_NETWORK_ENUM_UNKNOWN,		"unknown"},	/* fall though value */
 	{PK_NETWORK_ENUM_OFFLINE,		"offline"},
@@ -400,6 +366,14 @@ static const PkEnumMatch enum_upgrade_kind[] = {
 	{0, NULL}
 };
 
+static const PkEnumMatch enum_transaction_flag[] = {
+	{PK_TRANSACTION_FLAG_ENUM_NONE,		"none"},	/* fall though value */
+	{PK_TRANSACTION_FLAG_ENUM_ONLY_TRUSTED,	"only-trusted"},
+	{PK_TRANSACTION_FLAG_ENUM_SIMULATE,	"simulate"},
+	{PK_TRANSACTION_FLAG_ENUM_ONLY_DOWNLOAD, "only-download"},
+	{0, NULL}
+};
+
 /**
  * pk_enum_find_value:
  * @table: A #PkEnumMatch enum table of values
@@ -519,38 +493,6 @@ pk_distro_upgrade_enum_to_string (PkDistroUpgradeEnum upgrade)
 	return pk_enum_find_string (enum_upgrade, upgrade);
 }
 
-/**
- * pk_provides_enum_from_string:
- * @provides: Text describing the enumerated type
- *
- * Converts a text enumerated type to its unsigned integer representation
- *
- * Return value: the enumerated constant value, e.g. PK_PROVIDES_ENUM_MODALIAS
- *
- * Since: 0.5.0
- **/
-PkProvidesEnum
-pk_provides_enum_from_string (const gchar *provides)
-{
-	return pk_enum_find_value (enum_provides, provides);
-}
-
-/**
- * pk_provides_enum_to_string:
- * @provides: The enumerated type value
- *
- * Converts a enumerated type to its text representation
- *
- * Return value: the enumerated constant value, e.g. "codec"
- *
- * Since: 0.5.0
- **/
-const gchar *
-pk_provides_enum_to_string (PkProvidesEnum provides)
-{
-	return pk_enum_find_string (enum_provides, provides);
-}
-
 /**
  * pk_info_enum_from_string:
  * @info: Text describing the enumerated type
@@ -775,38 +717,6 @@ pk_restart_enum_to_string (PkRestartEnum restart)
 	return pk_enum_find_string (enum_restart, restart);
 }
 
-/**
- * pk_message_enum_from_string:
- * @message: Text describing the enumerated type
- *
- * Converts a text enumerated type to its unsigned integer representation
- *
- * Return value: the enumerated constant value, e.g. PK_SIGTYPE_ENUM_GPG
- *
- * Since: 0.5.0
- **/
-PkMessageEnum
-pk_message_enum_from_string (const gchar *message)
-{
-	return pk_enum_find_value (enum_message, message);
-}
-
-/**
- * pk_message_enum_to_string:
- * @message: The enumerated type value
- *
- * Converts a enumerated type to its text representation
- *
- * Return value: the enumerated constant value, e.g. "available"
- *
- * Since: 0.5.0
- **/
-const gchar *
-pk_message_enum_to_string (PkMessageEnum message)
-{
-	return pk_enum_find_string (enum_message, message);
-}
-
 /**
  * pk_group_enum_from_string:
  * @group: Text describing the enumerated type
@@ -998,3 +908,335 @@ pk_upgrade_kind_enum_to_string (PkUpgradeKindEnum upgrade_kind)
 {
 	return pk_enum_find_string (enum_upgrade_kind, upgrade_kind);
 }
+
+/**
+ * pk_transaction_flag_enum_from_string:
+ * @transaction_flag: Text describing the enumerated type
+ *
+ * Converts a text enumerated type to its unsigned integer representation
+ *
+ * Return value: the enumerated constant value, e.g. %PK_TRANSACTION_FLAG_ENUM_SIMULATE
+ *
+ * Since: 0.8.1
+ **/
+PkTransactionFlagEnum
+pk_transaction_flag_enum_from_string (const gchar *transaction_flag)
+{
+	return pk_enum_find_value (enum_transaction_flag, transaction_flag);
+}
+
+/**
+ * pk_transaction_flag_enum_to_string:
+ * @transaction_flag: The enumerated type value
+ *
+ * Converts a enumerated type to its text representation
+ *
+ * Return value: the enumerated constant value, e.g. "simulate"
+ *
+ * Since: 0.8.1
+ **/
+const gchar *
+pk_transaction_flag_enum_to_string (PkTransactionFlagEnum transaction_flag)
+{
+	return pk_enum_find_string (enum_transaction_flag, transaction_flag);
+}
+
+/**
+ * pk_info_enum_to_localised_text:
+ * @info: The enumerated type value
+ *
+ * Converts a enumerated type to its localized description
+ *
+ * Return Value: the translated text
+ *
+ * Since: 0.7.2
+ **/
+static const gchar *
+pk_info_enum_to_localised_text (PkInfoEnum info)
+{
+	const gchar *text = NULL;
+	switch (info) {
+	case PK_INFO_ENUM_LOW:
+		/* TRANSLATORS: The type of update */
+		text = _("Trivial");
+		break;
+	case PK_INFO_ENUM_NORMAL:
+		/* TRANSLATORS: The type of update */
+		text = dgettext("PackageKit", "Normal");
+		break;
+	case PK_INFO_ENUM_IMPORTANT:
+		/* TRANSLATORS: The type of update */
+		text = dgettext("PackageKit", "Important");
+		break;
+	case PK_INFO_ENUM_SECURITY:
+		/* TRANSLATORS: The type of update */
+		text = dgettext("PackageKit", "Security");
+		break;
+	case PK_INFO_ENUM_BUGFIX:
+		/* TRANSLATORS: The type of update */
+		text = dgettext("PackageKit", "Bug fix ");
+		break;
+	case PK_INFO_ENUM_ENHANCEMENT:
+		/* TRANSLATORS: The type of update */
+		text = dgettext("PackageKit", "Enhancement");
+		break;
+	case PK_INFO_ENUM_BLOCKED:
+		/* TRANSLATORS: The type of update */
+		text = dgettext("PackageKit", "Blocked");
+		break;
+	case PK_INFO_ENUM_INSTALLED:
+	case PK_INFO_ENUM_COLLECTION_INSTALLED:
+		/* TRANSLATORS: The state of a package */
+		text = dgettext("PackageKit", "Installed");
+		break;
+	case PK_INFO_ENUM_AVAILABLE:
+	case PK_INFO_ENUM_COLLECTION_AVAILABLE:
+		/* TRANSLATORS: The state of a package, i.e. not installed */
+		text = dgettext("PackageKit", "Available");
+		break;
+	default:
+		g_warning ("info unrecognised: %s", pk_info_enum_to_string (info));
+	}
+	return text;
+}
+
+/**
+ * pk_info_enum_to_localised_present:
+ * @info: The enumerated type value
+ *
+ * Converts a enumerated type to its localized description
+ *
+ * Return Value: the translated text
+ *
+ * Since: 0.7.2
+ **/
+const gchar *
+pk_info_enum_to_localised_present (PkInfoEnum info)
+{
+	const gchar *text = NULL;
+	switch (info) {
+	case PK_INFO_ENUM_DOWNLOADING:
+		/* TRANSLATORS: The action of the package, in present tense */
+		text = dgettext("PackageKit", "Downloading");
+		break;
+	case PK_INFO_ENUM_UPDATING:
+		/* TRANSLATORS: The action of the package, in present tense */
+		text = dgettext("PackageKit", "Updating");
+		break;
+	case PK_INFO_ENUM_INSTALLING:
+		/* TRANSLATORS: The action of the package, in present tense */
+		text = dgettext("PackageKit", "Installing");
+		break;
+	case PK_INFO_ENUM_REMOVING:
+		/* TRANSLATORS: The action of the package, in present tense */
+		text = dgettext("PackageKit", "Removing");
+		break;
+	case PK_INFO_ENUM_CLEANUP:
+		/* TRANSLATORS: The action of the package, in present tense */
+		text = dgettext("PackageKit", "Cleaning up");
+		break;
+	case PK_INFO_ENUM_OBSOLETING:
+		/* TRANSLATORS: The action of the package, in present tense */
+		text = dgettext("PackageKit", "Obsoleting");
+		break;
+	case PK_INFO_ENUM_REINSTALLING:
+		/* TRANSLATORS: The action of the package, in present tense */
+		text = dgettext("PackageKit", "Reinstalling");
+		break;
+	default:
+		text = pk_info_enum_to_localised_text (info);
+	}
+	return text;
+}
+
+/**
+ * pk_info_enum_to_localised_past:
+ * @info: The enumerated type value
+ *
+ * Converts a enumerated type to its localized description
+ *
+ * Return Value: the translated text
+ *
+ * Since: 0.7.2
+ **/
+const gchar *
+pk_info_enum_to_localised_past (PkInfoEnum info)
+{
+	const gchar *text = NULL;
+	switch (info) {
+	case PK_INFO_ENUM_DOWNLOADING:
+		/* TRANSLATORS: The action of the package, in past tense */
+		text = dgettext("PackageKit", "Downloaded");
+		break;
+	case PK_INFO_ENUM_UPDATING:
+		/* TRANSLATORS: The action of the package, in past tense */
+		text = dgettext("PackageKit", "Updated");
+		break;
+	case PK_INFO_ENUM_INSTALLING:
+		/* TRANSLATORS: The action of the package, in past tense */
+		text = dgettext("PackageKit", "Installed");
+		break;
+	case PK_INFO_ENUM_REMOVING:
+		/* TRANSLATORS: The action of the package, in past tense */
+		text = dgettext("PackageKit", "Removed");
+		break;
+	case PK_INFO_ENUM_CLEANUP:
+		/* TRANSLATORS: The action of the package, in past tense */
+		text = dgettext("PackageKit", "Cleaned up");
+		break;
+	case PK_INFO_ENUM_OBSOLETING:
+		/* TRANSLATORS: The action of the package, in past tense */
+		text = dgettext("PackageKit", "Obsoleted");
+		break;
+	case PK_INFO_ENUM_REINSTALLING:
+		/* TRANSLATORS: The action of the package, in past tense */
+		text = dgettext("PackageKit", "Reinstalled");
+		break;
+	default:
+		text = pk_info_enum_to_localised_text (info);
+	}
+	return text;
+}
+
+/**
+ * pk_role_enum_to_localised_present:
+ * @role: The enumerated type value
+ *
+ * Converts a enumerated type to its localized description
+ *
+ * Return Value: the translated text
+ *
+ * Since: 0.7.2
+ **/
+const gchar *
+pk_role_enum_to_localised_present (PkRoleEnum role)
+{
+	const gchar *text = NULL;
+	switch (role) {
+	case PK_ROLE_ENUM_UNKNOWN:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Unknown role type");
+		break;
+	case PK_ROLE_ENUM_DEPENDS_ON:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting dependencies");
+		break;
+	case PK_ROLE_ENUM_GET_UPDATE_DETAIL:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting update details");
+		break;
+	case PK_ROLE_ENUM_GET_DETAILS:
+	case PK_ROLE_ENUM_GET_DETAILS_LOCAL:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting details");
+		break;
+	case PK_ROLE_ENUM_REQUIRED_BY:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting requires");
+		break;
+	case PK_ROLE_ENUM_GET_UPDATES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting updates");
+		break;
+	case PK_ROLE_ENUM_SEARCH_DETAILS:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Searching by details");
+		break;
+	case PK_ROLE_ENUM_SEARCH_FILE:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Searching by file");
+		break;
+	case PK_ROLE_ENUM_SEARCH_GROUP:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Searching groups");
+		break;
+	case PK_ROLE_ENUM_SEARCH_NAME:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Searching by name");
+		break;
+	case PK_ROLE_ENUM_REMOVE_PACKAGES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Removing");
+		break;
+	case PK_ROLE_ENUM_INSTALL_PACKAGES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Installing");
+		break;
+	case PK_ROLE_ENUM_INSTALL_FILES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Installing files");
+		break;
+	case PK_ROLE_ENUM_REFRESH_CACHE:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Refreshing cache");
+		break;
+	case PK_ROLE_ENUM_UPDATE_PACKAGES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Updating packages");
+		break;
+	case PK_ROLE_ENUM_CANCEL:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Canceling");
+		break;
+	case PK_ROLE_ENUM_GET_REPO_LIST:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting repositories");
+		break;
+	case PK_ROLE_ENUM_REPO_ENABLE:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Enabling repository");
+		break;
+	case PK_ROLE_ENUM_REPO_SET_DATA:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Setting data");
+		break;
+	case PK_ROLE_ENUM_REPO_REMOVE:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Removing repository");
+		break;
+	case PK_ROLE_ENUM_RESOLVE:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Resolving");
+		break;
+	case PK_ROLE_ENUM_GET_FILES:
+	case PK_ROLE_ENUM_GET_FILES_LOCAL:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting file list");
+		break;
+	case PK_ROLE_ENUM_WHAT_PROVIDES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting provides");
+		break;
+	case PK_ROLE_ENUM_INSTALL_SIGNATURE:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Installing signature");
+		break;
+	case PK_ROLE_ENUM_GET_PACKAGES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting packages");
+		break;
+	case PK_ROLE_ENUM_ACCEPT_EULA:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Accepting EULA");
+		break;
+	case PK_ROLE_ENUM_DOWNLOAD_PACKAGES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Downloading packages");
+		break;
+	case PK_ROLE_ENUM_GET_DISTRO_UPGRADES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting upgrades");
+		break;
+	case PK_ROLE_ENUM_GET_CATEGORIES:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting categories");
+		break;
+	case PK_ROLE_ENUM_GET_OLD_TRANSACTIONS:
+		/* TRANSLATORS: The role of the transaction, in present tense */
+		text = dgettext("PackageKit", "Getting transactions");
+		break;
+	default:
+		g_warning ("role unrecognised: %s", pk_role_enum_to_string (role));
+	}
+	return text;
+}
